@@ -4,38 +4,47 @@ gitlab-trace: show the status/trace of a GitLab CI pipeline/job.
 """
 
 import argparse
-import sys
+import json
 import subprocess
+import sys
 import urllib.parse
+from typing import Iterable, List, Optional, TypeVar
 
 import colorama
 import gitlab
 
 
-__version__ = '0.3.0'
+__version__ = '0.4.0'
 __author__ = "Marius Gedminas <marius@gedmin.as>"
 
 
-def fatal(msg):
+T = TypeVar('T')
+
+
+def fatal(msg: str) -> None:
     sys.exit(msg)
 
 
-def warn(msg):
+def warn(msg: str) -> None:
     print(msg, file=sys.stderr)
 
 
-def info(msg):
+def info(msg: str) -> None:
     print(msg, file=sys.stderr)
 
 
-def first(seq, default=None):
+def first(seq: Iterable[T], default: Optional[T] = None) -> Optional[T]:
     return next(iter(seq), default)
 
 
-def determine_project(url=None):
+def pipe(command: List[str]) -> str:
+    return subprocess.run(command, stdout=subprocess.PIPE,
+                          universal_newlines=True).stdout.rstrip('\n')
+
+
+def determine_project(url: str = None) -> Optional[str]:
     if not url:
-        url = subprocess.run('git remote get-url origin'.split(),
-                             capture_output=True, text=True).stdout.strip()
+        url = pipe('git remote get-url origin'.split())
     if '://' not in url:
         return None
     if urllib.parse.urlparse(url).hostname in ('', 'github.com'):
@@ -46,12 +55,11 @@ def determine_project(url=None):
     return project
 
 
-def determine_branch():
-    return subprocess.run('git symbolic-ref HEAD --short'.split(),
-                          capture_output=True, text=True).stdout.strip()
+def determine_branch() -> str:
+    return pipe('git symbolic-ref HEAD --short'.split())
 
 
-def fmt_status(status):
+def fmt_status(status: str) -> str:
     colors = {
         'success': colorama.Fore.GREEN,
         'failed': colorama.Fore.RED,
@@ -65,7 +73,22 @@ def fmt_status(status):
     return colors[status] + status + colorama.Style.RESET_ALL
 
 
-def main():
+def fmt_duration(duration: Optional[float]) -> str:
+    if duration is None:
+        return 'n/a'
+    m, s = divmod(round(duration), 60)
+    h, m = divmod(m, 60)
+    bits = []
+    if h:
+        bits.append(f"{h}h")
+    if m:
+        bits.append(f"{m}m")
+    if s or not bits:
+        bits.append(f"{s}s")
+    return " ".join(bits)
+
+
+def main() -> None:
     colorama.init()
 
     parser = argparse.ArgumentParser(description=__doc__)
@@ -80,6 +103,10 @@ def main():
         "-v", "--verbose", action="store_true",
         help="print more information",
     )
+    parser.add_argument(
+        "--debug", action="store_true",
+        help="print even more information, for debugging",
+    )
     parser.add_argument(
         "-g", "--gitlab", metavar="NAME",
         help="select configuration section in ~/.python-gitlab.cfg",
@@ -165,14 +192,22 @@ def main():
                     args.job = found[-1]
                     info(f"Selecting the last one: {args.job}")
         if not args.job:
-            info(f"Available jobs for pipeline #{pipeline.id}:")
+            print(f"Available jobs for pipeline #{pipeline.id}:")
             for job in jobs:
                 status = fmt_status(job.status)
                 print(f"   --job={job.id} - {status} - {job.name}")
             sys.exit(0)
 
     job = project.jobs.get(args.job)
+    if args.verbose:
+        info(f"Job created:    {job.created_at}")
+        info(f"Job started:    {job.started_at or 'not yet'}")
+        info(f"Job finished:   {job.finished_at or 'not yet'}")
+        info(f"Job duration:   {fmt_duration(job.duration)}")
+    if args.debug:
+        info(json.dumps(job.attributes, indent=2))
     sys.stdout.buffer.write(job.trace())
+    sys.exit(0)
 
 
 if __name__ == "__main__":
